import 'dart:typed_data';

import 'package:flutter/material.dart';
import 'package:flutter_blurhash/flutter_blurhash.dart';
import 'package:image/image.dart' as graphics;
import 'package:transparent_image/transparent_image.dart';

const _DEFAULT_SIZE = 32;

/// Display a Hash then fade to Image
class BlurHash extends StatefulWidget {
  final VoidCallback onDecoded;

  const BlurHash({
    Key key,
    @required this.hash,
    this.color = Colors.blueGrey,
    this.imageFit = BoxFit.fill,
    this.decodingWidth = _DEFAULT_SIZE,
    this.decodingHeight = _DEFAULT_SIZE,
    this.image,
    this.onDecoded,
    this.fadeOutDuration = const Duration(milliseconds: 300),
    this.fadeOutCurve = Curves.easeOut,
    this.fadeInDuration = const Duration(milliseconds: 700),
    this.fadeInCurve = Curves.easeIn,
  })  : assert(color != null),
        assert(hash != null),
        assert(decodingWidth > 0),
        assert(decodingHeight != 0),
        super(key: key);

  final String hash;
  final Color color;
  final BoxFit imageFit;
  final int decodingWidth;
  final int decodingHeight;
  final String image;

  /// The duration of the fade-out animation for the [placeholder].
  final Duration fadeOutDuration;

  /// The curve of the fade-out animation for the [placeholder].
  final Curve fadeOutCurve;

  /// The duration of the fade-Out animation for the [image].
  final Duration fadeInDuration;

  /// The curve of the fade-in animation for the [image].
  final Curve fadeInCurve;

  @override
  _BlurHashState createState() => _BlurHashState();
}

class _BlurHashState extends State<BlurHash> {
  Future<Uint8List> _image;

  @override
  void initState() {
    super.initState();
    _decodeImage();
  }

  @override
  void didUpdateWidget(BlurHash oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.hash != oldWidget.hash ||
        widget.image != oldWidget.image ||
        widget.decodingWidth != oldWidget.decodingWidth ||
        widget.decodingHeight != oldWidget.decodingHeight) {
      _decodeImage();
    }
  }

  void _decodeImage() {
    _image = blurHashDecode(
      blurHash: widget.hash,
      width: widget.decodingWidth,
      height: widget.decodingHeight,
    ).then((rs) {
      final img = graphics.Image.fromBytes(
          widget.decodingWidth, widget.decodingHeight, rs);
      return Uint8List.fromList(graphics.encodePng(img));
    }).then((value) {
      widget.onDecoded?.call();
      return value;
    });
  }

  @override
  Widget build(BuildContext context) => Stack(
        fit: StackFit.expand,
        alignment: Alignment.center,
        children: [
          buildBlurHashBackground(),
          if (widget.image != null) buildDisplayedImage(),
        ],
      );

  Widget buildDisplayedImage() => FadeInImage.memoryNetwork(
      fadeInCurve: widget.fadeInCurve,
      fadeInDuration: widget.fadeInDuration,
      fadeOutCurve: widget.fadeOutCurve,
      fadeOutDuration: widget.fadeOutDuration,
      fit: widget.imageFit,
      image: widget.image,
      placeholder: kTransparentImage);

  Widget buildBlurHashBackground() => FutureBuilder<Uint8List>(
        future: _image,
        builder: (ctx, snap) => snap.hasData
            ? Image(image: MemoryImage(snap.data), fit: widget.imageFit)
            : Container(color: widget.color),
      );
}
